
local json = require("json")
local utils = require("utils")
local utils_device = require("utils_device")

function insert_factor(list, typ, profile, profile_key)
    if ("All" == factor_type or typ == factor_type) then
        for _, factor in ipairs(profile[profile_key]) do

            factor_id = factor["identifier"]
            -- get and set service is the implicit services
            if ("set" ~= factor_id and "get" ~= factor_id) then
                item = {}
                item["FactorType"] = typ
                item["FactorName"] = factor_id
                item["FactorId"] = factor_id
                rw = factor["accessMode"]
                if ('r' == rw) then
                    rw = "ReadOnly"
                elseif ('w' == rw) then
                    rw = "WriteOnly"
                else
                    rw = "RW"
                end 
                item["Rw"] = rw
                table.insert(list, item)
            end
        end
    end
end


local request_method = ngx.var.request_method

utils.check_login()
utils.check_permission("DeviceManage", "GatewayManage")

if ("GET" == request_method) then
    local args = ngx.req.get_uri_args()
    if (nil == args) then
        utils.error_exit(400, "invalid parameter")
    end

    pk = args["ProductKey"]
    dn = args["DeviceName"]
    factor_type = args["FactorType"]
    page_size = tonumber(args["PageSize"])
    cur_page = tonumber(args["CurrentPage"])

    utils.check_pk(pk)
    utils.check_dn(dn)
    utils.check_page_size(page_size)
    utils.check_current_page(cur_page)

    if (not (factor_type and ("All" == factor_type or "Service" == factor_type or "Property" == factor_type or "Event" == factor_type))) then
        utils.error_exit(400, "invalid value of FactorType key")
    end

    local ok, profile = utils_device.get_device_profile(pk, dn)
    if (nil == profile) then
        if (ok) then
            utils.error_exit(200)
        else
            utils.error_exit(400, "no "..pk.." profile found , please import this.")
        end
    end

    local factor_list = {}
    insert_factor(factor_list, "Property", profile, "properties")
    insert_factor(factor_list, "Service", profile, "services")
    insert_factor(factor_list, "Event", profile, "events")

    factor_list_res = {}
    table.sort(factor_list, function (e1, e2)
        return e1["FactorName"] <= e2["FactorName"]
        end)
    min, max = utils.get_item_index(#factor_list, page_size, cur_page)
    for i = min, max do
        table.insert(factor_list_res, factor_list[i])
    end

    res = {}
    res["TotalNum"] = #factor_list
    res["PageSize"] = page_size
    res["CurrentPage"] = cur_page
    res["List"] = factor_list_res
    res_raw = json.encode(res)
    utils.error_exit(200, res_raw)
end

utils.error_exit(400, "Invalid request method")
