#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys

sys.path.append("../../lib/")

from leda_python import leda
from leda_python.mbus import MbusBase
import hashlib
import logging
import threading
import json
import time
import os
import unittest

_logger = logging.getLogger(__name__)

driverObj = leda.LedaModule()


class Demo_device(leda.BaseDeviceCallback):
    def __init__(self):
        self.name = "jys"

    def callService(self, name, input):
        pass

    def getprofile_cb(self):
        pass

    def getProperties(self, input):
        pass

    def setProperties(self, input):
        pass


def createThread(task, *args):
    t = threading.Thread(target=task, args=args)
    t.setDaemon(True)
    t.start()


def config_stop():
    os.system(
        "ps -ef | grep \"ut_config\" | grep -v \"grep\" | awk \'{print $2}\' | xargs kill -2 1>> /dev/null 2>>/dev/null")


def config_start():
    config_stop()
    os.system("./startConfig.sh & 1>> /dev/null 2>>/dev/null")
    time.sleep(3)


def configTimeOut_start():
    config_stop()
    os.system("./startConfig_timeOut.sh &  1>> /dev/null 2>>/dev/null")
    time.sleep(3)


def configErrCode_start():
    config_stop()
    os.system("./startConfig_errCode.sh & 1>> /dev/null 2>>/dev/null")
    time.sleep(3)


def dimu_stop():
    os.system(
        "ps -ef | grep \"ut_dimu\" | grep -v \"grep\" | awk \'{print $2}\' | xargs kill -2 1>> /dev/null 2>>/dev/null")


def dimu_start():
    dimu_stop()
    os.system("./startDimu.sh & 1>> /dev/null 2>>/dev/null")
    time.sleep(3)


def dimuTimeOut_start():
    dimu_stop()
    os.system("./startDimu_timeOut.sh & 1>> /dev/null 2>>/dev/null")
    time.sleep(3)


def dimuErrCode_start():
    dimu_stop()
    os.system("./startDimu_errCode.sh & 1>> /dev/null 2>>/dev/null")
    time.sleep(3)


def dimuRetParamsInvalid_start():
    dimu_stop()
    os.system("./startDimu_retParamsInvalid.sh & 1>> /dev/null 2>>/dev/null")
    time.sleep(3)


class SDK_unittest(unittest.TestCase):

    def tearDown(self):
        driverObj._releaseMbusObject()

    # @unittest.skip('i dont test the case')
    def test_registerDriver_invalidDriverName(self):

        dimu_start()
        driverObj.moduleInit('abc123')

        driver_name_list = [123, '12abc', 'ac#@wed',
                            'a1234567890123456789012345678901234567890123456789012345678901234', '']

        for driver_name in driver_name_list:

            try:
                driverObj.moduleInit(driver_name)  # type of driver_name is invalid
                print("test: moduleInit inparam: driver_name(%s) ...Err" % driver_name)
                myAssertID = False
            except:
                print("test: moduleInit inparam: driver_name(%s) ...Ok" % driver_name)
                myAssertID = True

            self.assertEqual(myAssertID, True)

    # @unittest.skip('i dont test the case')
    def test_registerDriver_rightDriverName(self):
        driver_name = "abc123"
        dimu_start()

        try:
            driverObj.moduleInit(driver_name)
            assertID = True
        except:
            _logger.exception('Err')
            assertID = False

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_registerDriver_timeOut(self):
        dimuTimeOut_start()
        driver_name = "abc123"
        try:
            driverObj.moduleInit(driver_name)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_registerDriver_errCode(self):
        dimuErrCode_start()
        driver_name = "abc123"
        try:
            driverObj.moduleInit(driver_name)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_registerDriver_retParamsInvalid(self):
        dimuRetParamsInvalid_start()
        driver_name = "abc123"
        try:
            driverObj.moduleInit(driver_name)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_unregisterDriver(self):
        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)

        try:

            driverObj._unregisterModule()
            assertID = True
        except:
            assertID = False

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_unregisterDriver_timeout(self):

        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)

        try:
            dimuTimeOut_start()
            driverObj._unregisterModule()
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_unregisterDriver_errCode(self):

        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)

        try:
            dimuErrCode_start()
            driverObj._unregisterModule()
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_unregisterDriver_retParamsInvalid(self):

        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)

        try:
            dimuRetParamsInvalid_start()
            driverObj._unregisterModule()
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_registerDevice_timeout(self):
        dimu_start()
        driver_name = "abc123"
        device_name = 'device123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        bus_callback_object = Demo_device()

        driverObj.moduleInit(driver_name)

        dimuTimeOut_start()
        try:
            driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_registerDevice_errCode(self):
        dimu_start()
        driver_name = "abc123"
        device_name = 'device123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        bus_callback_object = Demo_device()

        driverObj.moduleInit(driver_name)

        dimuErrCode_start()

        try:
            driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_registerDevice_deviceCallBackObjErr(self):
        driver_name = "abc123"
        device_name = 'device123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class cmp_bus_test(object):
            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)

        try:
            bus_callback_object = cmp_bus_test()
            driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_registerDevice_invalidDevName(self):

        driver_name = "abc123"
        device_name = 123
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)

        try:
            bus_callback_object = Demo_device()
            driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_registerDevice_invalidDevName2(self):
        driver_name = "abc123"
        device_name = "12de%$#"
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)

        try:
            bus_callback_object = Demo_device()
            driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_registerDevice_invalidDevName3(self):
        driver_name = "abc123"
        device_name = "abc1234567890123456789012345678901234567890123456789012345678901234567890"
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)

        try:
            bus_callback_object = Demo_device()
            driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_registerDevice_invalidPK(self):
        driver_name = "abc123"
        device_name = "abc123"
        product_key = 123

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)

        try:
            bus_callback_object = Demo_device()
            driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_registerDevice_invalidProfile(self):

        driver_name = "abc123"
        device_name = "abc123"
        product_key = 'abc123'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)

        try:
            profile = 123
            bus_callback_object = Demo_device()
            driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_registerDevice_repeat(self):

        driver_name = "abc123"
        device_name = "abc123"
        product_key = 'abc123'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)

        try:
            bus_callback_object = Demo_device()
            driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
            driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
            assertID = True
        except:
            assertID = False

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_registerDevice_oldAPI(self):

        driver_name = "abc123"
        device_name = 123
        product_key = 'abc123'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)

        try:
            bus_callback_object = Demo_device()
            driverObj.driver_register_device(device_name, 'md5', product_key, profile, bus_callback_object)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_UnregisterDevice_inparamsTypeInvalid(self):
        dimu_start()
        driver_name = "abc123"
        device_name = 'device123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        bus_callback_object = Demo_device()

        driverObj.moduleInit(driver_name)
        driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)

        try:
            driverObj.deviceUnregister(12)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_UnregisterDevice_PKInvalid(self):
        dimu_start()
        driver_name = "abc123"
        device_name = 'device123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        bus_callback_object = Demo_device()

        driverObj.moduleInit(driver_name)
        dev = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
        dev.product_key = 'rfv'
        try:
            driverObj.deviceUnregister(dev)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_UnregisterDevice_timeout(self):
        dimu_start()
        driver_name = "abc123"
        device_name = 'device123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        bus_callback_object = Demo_device()

        driverObj.moduleInit(driver_name)
        dev = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
        dev.offline()
        dimuTimeOut_start()
        driverObj.deviceUnregister(dev)

        self.assertEqual(True, True)

    # @unittest.skip('i dont test the case')
    def test_UnregisterDevice_ok(self):
        dimu_start()
        driver_name = "abc123"
        device_name = 'device123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        bus_callback_object = Demo_device()

        driverObj.moduleInit(driver_name)
        dev = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
        driverObj.deviceUnregister(dev)

        self.assertEqual(True, True)

    # @unittest.skip('i dont test the case')
    def test_UnregisterDevice_retParamsInvalid(self):
        dimu_start()
        driver_name = "abc123"
        device_name = 'device123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        bus_callback_object = Demo_device()

        driverObj.moduleInit(driver_name)
        dev = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)

        dimuRetParamsInvalid_start()
        driverObj.deviceUnregister(dev)

        self.assertEqual(True, True)

    # @unittest.skip('i dont test the case')
    def test_UnregisterDevice_errCode(self):
        dimu_start()
        driver_name = "abc123"
        device_name = 'device123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        bus_callback_object = Demo_device()

        driverObj.moduleInit(driver_name)
        dev = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)

        dimuErrCode_start()

        driverObj.deviceUnregister(dev)

        self.assertEqual(True, True)

    # @unittest.skip('i dont test the case')
    def test_watchdog_argsTypeError(self):
        dimu_start()

        driver_name = "abc123"
        driverObj.moduleInit(driver_name)

        try:
            thread_name = 123
            count_down = 10
            driverObj.feedDog(thread_name, count_down)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_watchdog_emptyArgs(self):
        dimu_start()

        driver_name = "abc123"
        driverObj.moduleInit(driver_name)

        try:
            thread_name = ''
            count_down = 10
            driverObj.feedDog(thread_name, count_down)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_watchdog_errArgs(self):
        dimu_start()

        driver_name = "abc123"
        driverObj.moduleInit(driver_name)

        try:
            thread_name = "thread123"
            count_down = -123
            driverObj.feedDog(thread_name, count_down)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_watchdog_errArgs2(self):
        dimu_start()

        driver_name = "abc123"
        driverObj.moduleInit(driver_name)

        try:
            thread_name = "thread123"
            count_down = '123'
            driverObj.feedDog(thread_name, count_down)
            assertID = False
        except:
            _logger.exception('Err')
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_watchdog_ok(self):
        dimu_start()

        driver_name = "abc123"
        driverObj.moduleInit(driver_name)

        try:
            driverObj.feedDog("thread_name", 123)
            assertID = True
        except:
            assertID = False

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_online_ok(self):

        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)

        device_name = 'device123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        bus_callback_object = Demo_device()
        try:
            deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
            assertID = True
        except:
            print("device register:...Err")
            assertID = False

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_online_timeout(self):

        driver_name = "abc123"
        device_name = 'abc123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)
        bus_callback_object = Demo_device()
        deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)

        dimuTimeOut_start()
        try:
            deviceObj.online()
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_online_errCode(self):
        driver_name = "abc123"
        device_name = 'abc123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)
        bus_callback_object = Demo_device()
        deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)

        dimuErrCode_start()
        try:
            deviceObj.online()
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_online_retParamsInvalid(self):
        driver_name = "abc123"
        device_name = 'abc123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)
        bus_callback_object = Demo_device()
        deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)

        dimuRetParamsInvalid_start()
        try:
            deviceObj.online()
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_offline_timeout(self):
        driver_name = "abc123"
        device_name = 'abc123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)
        bus_callback_object = Demo_device()
        deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
        deviceObj.online()
        dimuTimeOut_start()

        try:
            deviceObj.offline()
            assertID = True
        except:
            assertID = False

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_offline_errCode(self):
        driver_name = "abc123"
        device_name = 'abc123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)
        bus_callback_object = Demo_device()
        deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
        deviceObj.online()

        dimuErrCode_start()
        try:
            deviceObj.offline()
            assertID = True
        except:
            assertID = False

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_offline_retParamsInvalid(self):
        driver_name = "abc123"
        device_name = 'abc123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)
        bus_callback_object = Demo_device()
        deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
        deviceObj.online()

        dimuRetParamsInvalid_start()
        try:
            deviceObj.offline()
            assertID = True
        except:
            assertID = False

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_reportProperty_beforeOnline(self):
        driver_name = "abc123"
        device_name = 'abc123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)
        bus_callback_object = Demo_device()
        deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)

        properties_dict = {
            "propertyName1": "xxx",
            "propertyName2": "yyy",
            "propertyName3": "zzz"
        }

        try:
            deviceObj.offline()
            deviceObj.reportProperties(properties_dict)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_reportProperty_ok(self):
        driver_name = "abc123"
        device_name = 'abc123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)
        bus_callback_object = Demo_device()
        deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)

        properties_dict = {
            "propertyName1": "xxx",
            "propertyName2": "yyy",
            "propertyName3": "zzz"
        }

        try:
            deviceObj.reportProperties(properties_dict)
            assertID = True
        except:
            assertID = False

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_reportProperty_invalidDataType(self):
        driver_name = "abc123"
        device_name = 'abc123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)
        bus_callback_object = Demo_device()
        deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
        deviceObj.online()

        properties_dict = 123

        try:
            deviceObj.reportProperties(properties_dict)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_reportProperty_invalidDataType2(self):
        driver_name = "abc123"
        device_name = 'abc123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)
        bus_callback_object = Demo_device()
        deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
        deviceObj.online()

        properties_dict = 123

        try:
            deviceObj.device_report_property(properties_dict)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_reportEvent_beforOnline(self):
        driver_name = "abc123"
        device_name = 'abc123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)
        bus_callback_object = Demo_device()
        deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
        event_dict = {
            "eventArgs1": "xxx",
            "eventArgs2": "yyy",
            "eventArgs3": "zzz"
        }

        try:
            deviceObj.offline()
            deviceObj.reportEvent('eventName', event_dict)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_reportEvent_ok(self):
        driver_name = "abc123"
        device_name = 'abc123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)
        bus_callback_object = Demo_device()
        deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
        event_dict = {
            "eventArgs1": "xxx",
            "eventArgs2": "yyy",
            "eventArgs3": "zzz"
        }

        try:
            deviceObj.reportEvent('eventName', event_dict)
            assertID = True
        except:
            assertID = False

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_reportEvent_invalidArgs0(self):
        driver_name = "abc123"
        device_name = 'abc123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)
        bus_callback_object = Demo_device()
        deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
        event_dict = {
            "eventArgs1": "xxx",
            "eventArgs2": "yyy",
            "eventArgs3": "zzz"
        }

        try:
            deviceObj.device_report_event('eventName', event_dict)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_reportEvent_invalidArgs(self):
        driver_name = "abc123"
        device_name = 'abc123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)
        bus_callback_object = Demo_device()
        deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
        deviceObj.online()
        event_dict = {
            "eventArgs1": "xxx",
            "eventArgs2": "yyy",
            "eventArgs3": "zzz"
        }

        try:
            deviceObj.reportEvent(123, event_dict)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_reportEvent_invalidArgs2(self):
        driver_name = "abc123"
        device_name = 'abc123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        dimu_start()
        driverObj.moduleInit(driver_name)
        bus_callback_object = Demo_device()
        deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
        deviceObj.online()
        event_dict = {
            "eventArgs1": "xxx",
            "eventArgs2": "yyy",
            "eventArgs3": "zzz"
        }

        try:
            name = 'abcded12345678901234567890123456789012345678901234567890123456789012345678901234567890'
            deviceObj.reportEvent(name, event_dict)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_GetPdInfo_ok(self):

        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)

        config_start()
        try:
            driverObj.getPdInfo()
            assertID = True
        except:
            assertID = False

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_GetPdInfo_inparamsTypeInvalid(self):

        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)

        config_start()
        try:
            driverObj.getConfig(234)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_GetInfo_errCode(self):

        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)

        configErrCode_start()

        try:
            driverObj.getPdInfo()
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_GetInfo_timeOut(self):

        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)
        configTimeOut_start()

        try:
            driverObj.getPdInfo()
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_SubConfig_ok(self):

        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)

        class ConfigCallback(leda.LedaConfigCallback):
            def deviceConfigCB(self, moduleName, moduleConfig):
                print("configCallback: moduleName %s, moduleConfigMsg...ok!!: %s", moduleName, moduleConfig)

        config_start()

        try:
            driverObj.registerDeviceConfigCallback(ConfigCallback())
            assertID = True
        except:
            assertID = False

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_SubConfig_errCode(self):
        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)

        class ConfigCallback(leda.LedaConfigCallback):
            def deviceConfigCB(self, moduleName, moduleConfig):
                print("configCallback: moduleName %s, moduleConfigMsg...ok!!: %s", moduleName, moduleConfig)

        configErrCode_start()

        try:
            driverObj.registerDeviceConfigCallback(ConfigCallback())
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_SubConfig_inparamsInvalid(self):
        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)
        config_start()
        try:
            driverObj.registerDeviceConfigCallback(123)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_SubConfig_inparamsInvalid2(self):
        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)
        config_start()
        try:
            driverObj.subConfig(123)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_SubConfig_timeOut(self):

        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)

        class ConfigCallback(leda.LedaConfigCallback):
            def deviceConfigCB(self, moduleName, moduleConfig):
                print("configCallback: moduleName %s, moduleConfigMsg...ok!!: %s", moduleName, moduleConfig)

        configTimeOut_start()

        try:
            driverObj.registerDeviceConfigCallback(ConfigCallback())
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_register_500devices(self):
        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)

        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        bus_callback_object = Demo_device()

        tmpName = 'deviceaabb'
        cnt_ok = 0
        cnt_fail = 0

        for i in range(0, 500):
            device_name = tmpName + str(i)
            try:

                deviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
                cnt_ok += 1
                if (cnt_ok == 200):
                    dimuErrCode_start()
                    print("<><><><><><><><><><><><>")
            except:
                cnt_fail += 1

        self.assertEqual(cnt_ok, 200)
        self.assertEqual(cnt_fail, 300)

    # @unittest.skip('i dont test the case')
    def test_reportProperties_40Devices(self):
        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)

        device_name = 'device123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        tmpName = 'deviceaabb'
        deviceObj = []
        bus_callback_object = Demo_device()
        properties_dict = {
            "propertyName1": "xxx",
            "propertyName2": "yyy",
            "propertyName3": "zzz"
        }

        for i in range(0, 40):
            device_name = tmpName + str(i)
            try:
                TmpdeviceObj = driverObj.deviceRegister(device_name, product_key, profile,
                                                        bus_callback_object)

                TmpdeviceObj.online()
                deviceObj.append(TmpdeviceObj)
            except:
                print("device register: Err")

        def Task_reportProperty(deviceObj, properties_dict):

            for i in range(0, 10):
                deviceObj.reportEvent("event", properties_dict)
                time.sleep(0.5)

            thread = threading.current_thread()
            print("%s report property cnt: 10...Ok" % (thread.getName()))

        for i in range(0, 40):
            createThread(Task_reportProperty, deviceObj[i], properties_dict)

        time.sleep(6)

        self.assertEqual(1, 1)

    # @unittest.skip('i dont test the case')
    def test_reportEvent_40devices(self):

        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)

        device_name = 'device123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        tmpName = 'deviceaabb'
        deviceObj = []
        bus_callback_object = Demo_device()
        event_dict = {
            "key1": "value1",
            "key2": "value2"
        }

        for i in range(0, 40):
            device_name = tmpName + str(i)
            try:
                TmpdeviceObj = driverObj.deviceRegister(device_name, product_key, profile,
                                                        bus_callback_object)

                TmpdeviceObj.online()
                deviceObj.append(TmpdeviceObj)
            except:
                print("device register:Err")

        def Task_reportEvent(deviceObj, event_dict):

            for i in range(0, 10):
                deviceObj.reportEvent("event", event_dict)
                time.sleep(0.5)

            thread = threading.current_thread()
            print("%s report event cnt: 10...ok" % (thread.getName()))

        for dev in deviceObj:
            createThread(Task_reportEvent, dev, event_dict)

        time.sleep(6)

        self.assertEqual(1, 1)

    # @unittest.skip('i dont test the case')
    def test_online_40devices(self):
        driver_name = "abc123"
        dimu_start()
        driverObj.moduleInit(driver_name)

        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        bus_callback_object = Demo_device()

        tmpName = 'deviceaabb'
        cnt_ok = 0
        cnt_fail = 0
        deviceObj = []

        for i in range(0, 40):
            device_name = tmpName + str(i)
            try:
                TmpdeviceObj = driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
                deviceObj.append(TmpdeviceObj)
            except:
                print("device register: Err")
        for dev in deviceObj:

            try:
                dev.online()
                cnt_ok += 1
            except:
                cnt_fail += 1

        self.assertEqual(cnt_ok, 40)


if (__name__ == "__main__"):
    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s-%(threadName)-9s-%(filename)s[line:%(lineno)d]-%(levelname)s: %(message)s')

    _logger.debug("demo driver testing...")

    # suite = unittest.TestSuite()#创建测试套件
    unittest.main(verbosity=2)
